/*
%%%-------------------------------------------------------------------
%%% File    : syb_demo.c
%%% Author  : MP Scholtz (michael@erlfinsys.net)
%%% Description : Top level C functions for Erlang Sybase C Driver
%%%
%%% Modified : 2005/08/16
%%% Original file : wide_rpc.c (From ctlibrary samples)
%%%-------------------------------------------------------------------
*/




#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctpublic.h>
#include "example.h"
#include "exutils.h"

#if !lint
static char Sccsid[] = {"%Z% %M% %I% %G%"};
#endif /* !lint */

/*****************************************************************************
** 
** defines and globals used.
** 
*****************************************************************************/
/*
** Global names used in this module
*/

CS_COMMAND	*cmd;
CS_CONNECTION   *connection;
CS_CONTEXT	*context;
/*
** Prototypes for routines in the example code.
*/
CS_STATIC CS_RETCODE InstallNulls PROTOTYPE((
	));
CS_STATIC CS_RETCODE BuildRpcCommand PROTOTYPE((
						   char *arg, int bufflen
	));
CS_STATIC CS_RETCODE DoRpc PROTOTYPE((
					 char *arg, int bufflen, example_data* d
	));



char doStoredProc(char *arg, int bufflen, example_data* d)
{
    CS_RETCODE retcode;



	/*
	** Execute the the newly created RPC
	*/
    retcode = DoRpc(arg, bufflen, d);
		
    /*  return (retcode == CS_SUCCEED) ? EX_EXIT_SUCCEED : EX_EXIT_FAIL;*/
    return 0;
}

char connect1(char *arg, int arglen)
{
    int i=0;
    
    CS_CHAR *Ex_appname;
    CS_CHAR *Ex_dbname;
    CS_CHAR *Ex_server;
    CS_CHAR *Ex_username;
    CS_CHAR *Ex_password;
    
    CS_RETCODE	retcode;
	
    EX_SCREEN_INIT();


    Ex_appname = &arg[i];
    while(arg[i] != 0)
    {
	i++;
    }
    i++;
    Ex_dbname = &arg[i];
    while(arg[i] != 0)
    {
	i++;
    }
    i++;
    Ex_server = &arg[i];
    while(arg[i] != 0)
    {
	i++;
    }
    i++;	
    Ex_username = &arg[i];
    while(arg[i] != 0)
    {
	i++;
    }
    i++;   
    Ex_password = &arg[i];
   
	/* 
	** Allocate a context structure and initialize Client-Library 
	*/
	retcode = ex_init(&context);
	if (retcode != CS_SUCCEED)
	{
	    ex_panic("ex_init failed");
	}

	/* 
	** Allocate a connection structure, set its properties, and  
	** establish a connection.
	*/
	retcode = ex_connect(context, &connection, Ex_appname,
			     Ex_username, Ex_password, Ex_server);
	if (retcode == CS_SUCCEED)
	{
	    retcode = InstallNulls(context);
	}
	/*
	** Use the database
	*/
	if (retcode == CS_SUCCEED)
	{
	    if ((retcode = ex_use_db(connection, Ex_dbname)) != CS_SUCCEED)
	    {
                ex_error("DoRpc: ex_use_db() failed");
                return retcode;
	    }
	}
	if ((retcode = ct_cmd_alloc(connection, &cmd)) != CS_SUCCEED)
        {
                ex_error("DoRpc: ct_cmd_alloc() failed");
                return retcode;
        }
       
	if (retcode != CS_SUCCEED)
	{
	    return 1;
	}else {
	    return 0;
	}
}



/*
** InstallNulls()
**
** Type of function:
** 	rpc program internal api
**
** Purpose:
** 	Install the character string "NULL" as the default null value
**	when bind to character columns.
**
** Parameters:
** 	context	- Pointer to CS_CONTEXT structure.
**
** Return:
**	CS_SUCCEED if null was installed.
**	Otherwise a Client-Library failure code.
*/

CS_STATIC CS_RETCODE 
InstallNulls()
{
	CS_DATAFMT	datafmt;

	memset(&datafmt, 0, sizeof (datafmt));
	datafmt.datatype = CS_CHAR_TYPE;
	datafmt.status = CS_FMT_UNUSED;
	datafmt.locale = NULL;
	
	return cs_setnull(context, &datafmt, "NULL", strlen("NULL"));
}

/*
** BuildRpcCommand()
**
** Type of function:
** 	rpc program internal api
**
** Purpose:
**	This routine contructs the parameters list for the rpc to execute
**
** Parameters:
** 	cmd	- Pointer to CS_COMMAND structure.
**
** Return:
**	CS_SUCCEED if rpc command was contructed
**	Otherwise a Client-Library failure code.
** 
*/

CS_STATIC CS_RETCODE 
BuildRpcCommand(arg, bufflen)
char *arg;
int bufflen;
{
    int i = 0;
        CS_RETCODE	retcode;
	CS_DATAFMT	datafmt;
	CS_DATAFMT	srcfmt;
	CS_DATAFMT	destfmt;
	CS_INT		intvar;
	CS_MONEY	moneyvar;
	CS_CHAR         charvar;
	CS_DATETIME     datetimevar;
	char		rpc_name[50];
	char            *argstring;
	char            argtype;
	CS_INT		destlen;

	/*
	** Get Rpc Name from args.
	*/	
	while(arg[i] != 0)
	{
	    i++;
	}
	i++;

	strcpy(rpc_name, arg);


	if ((retcode = ct_cmd_props(cmd, CS_GET, CS_PARENT_HANDLE,
				&connection, CS_UNUSED, NULL)) != CS_SUCCEED)
	{
                ex_error("BuildRpcCommand: ct_cmd_props() failed");
		return retcode;
	}
	if ((retcode = ct_con_props(connection, CS_GET, CS_PARENT_HANDLE,
				&context, CS_UNUSED, NULL)) != CS_SUCCEED)
	{
                ex_error("BuildRpcCommand: ct_con_props() failed");
		return retcode;
		}

	/*
	** Send the RPC command for our stored procedure.
	*/
	if ((retcode = ct_command(cmd, CS_RPC_CMD, rpc_name, CS_NULLTERM,
			CS_NO_RECOMPILE)) != CS_SUCCEED)
	{
                ex_error("BuildRpcCommand: ct_command() failed");
		return retcode;
	}	CS_BINARY	binaryvar;


	/*
	** Get each parameter value and type, do conversion and pass the converted value
	** to Sybase.
	*/
	while(i<bufflen)
	{
	    argstring = &arg[i];
	    while(arg[i] != 0)
	    {
		i++;
	    }
	    i++;
	    argtype = arg[i];
	    i++;
	    i++;
	    switch (argtype)
	    {
		case  12:
		    memset(&srcfmt, 0, sizeof (CS_DATAFMT));
		    srcfmt.datatype = CS_CHAR_TYPE;
		    srcfmt.maxlength = strlen(argstring);
		    srcfmt.locale = NULL;

		    memset(&destfmt, 0, sizeof (CS_DATAFMT));
		    destfmt.datatype = CS_INT_TYPE;
		    destfmt.maxlength = sizeof(CS_INT);
		    destfmt.locale = NULL;

		    retcode = cs_convert(context, &srcfmt, (CS_VOID *)argstring,
					 &destfmt, &intvar, &destlen);
		    if (retcode != CS_SUCCEED)
		    {
			ex_error("BuildRpcCommand: cs_convert() failed on integer conversion");
			return retcode;
		    }
		    
		    memset(&datafmt, 0, sizeof (datafmt));
		    datafmt.namelen = CS_NULLTERM;
		    datafmt.datatype = CS_INT_TYPE;
		    datafmt.maxlength = CS_UNUSED;
		    datafmt.status = CS_INPUTVALUE;
		    datafmt.locale = NULL;
		    
		    if ((retcode = ct_param(cmd, &datafmt, (CS_VOID *)&intvar, 
					    CS_SIZEOF(CS_INT), CS_UNUSED)) != CS_SUCCEED)
		    {
			ex_error("BuildRpcCommand: ct_param(int) failed");
			return retcode;
			}
		    break;
		    
		case  20:
		    memset(&datafmt, 0, sizeof (datafmt));
		    datafmt.namelen = CS_NULLTERM;
		    datafmt.datatype = CS_CHAR_TYPE;
		    datafmt.maxlength = 300;
		    datafmt.status = CS_INPUTVALUE;
		    datafmt.locale = NULL;
		    
		    if((retcode = ct_param(cmd, &datafmt, (CS_VOID *)argstring, strlen(argstring), CS_UNUSED))  != CS_SUCCEED)
		    {
			ex_error("BuildRpcCommand: ct_param(char) failed");
			return retcode;
		    }
		    break;
		    
		case  30:
		    memset(&srcfmt, 0, CS_SIZEOF(CS_DATAFMT));
		    srcfmt.datatype = CS_CHAR_TYPE;
		    srcfmt.maxlength = strlen(argstring);
		    srcfmt.locale = NULL;
		    srcfmt.format = CS_FMT_NULLTERM;

		    memset(&destfmt, 0, CS_SIZEOF(CS_DATAFMT));
		    destfmt.format = CS_FMT_UNUSED;
		    destfmt.datatype = CS_DATETIME_TYPE;
		    destfmt.maxlength = CS_SIZEOF(CS_DATETIME);
		    destfmt.locale = NULL;

		    retcode = cs_convert(context, &srcfmt, (CS_VOID *)argstring,
					 &destfmt, &datetimevar, &destlen);
		    if (retcode != CS_SUCCEED)
		    {
			ex_error("BuildRpcCommand: cs_convert() failed on date conversion");
			return retcode;
		    }
		    memset(&datafmt, 0, sizeof (datafmt));
		    datafmt.namelen = CS_NULLTERM;
		    datafmt.datatype = CS_DATETIME_TYPE;
		    datafmt.maxlength = CS_SIZEOF(CS_DATETIME);
		    datafmt.status = CS_INPUTVALUE;
		    datafmt.locale = NULL;
		    if((retcode = ct_param(cmd, &datafmt, (CS_VOID *)&datetimevar, CS_SIZEOF(CS_DATETIME), CS_UNUSED))  != CS_SUCCEED)
		    {
			ex_error("BuildRpcCommand: ct_param(datetime) failed");
			return retcode;
			}
		    break;
		    
		case 40:
		    memset(&srcfmt, 0, sizeof (CS_DATAFMT));
		    srcfmt.datatype = CS_CHAR_TYPE;
		    srcfmt.maxlength = strlen(argstring);
		    srcfmt.precision = 5;
		    srcfmt.scale = 2;
		    srcfmt.locale = NULL;
		    
		    memset(&destfmt, 0, sizeof (CS_DATAFMT));
		    destfmt.datatype = CS_MONEY_TYPE;
		    destfmt.maxlength = sizeof(CS_MONEY);
		    destfmt.precision = 5;
		    destfmt.scale = 2;
		    destfmt.locale = NULL;


		    retcode = cs_convert(context, &srcfmt, (CS_VOID *)argstring,
					 &destfmt, &moneyvar, &destlen);
		    if (retcode != CS_SUCCEED)
		    {
			ex_error("BuildRpcCommand: cs_convert() failed on money conversion");
			return retcode;
		    }

		    memset(&datafmt, 0, sizeof (datafmt));
		    datafmt.namelen = CS_NULLTERM;
		    datafmt.datatype = CS_MONEY_TYPE;
		    datafmt.maxlength = EX_MAXSTRINGLEN;
		    datafmt.status = CS_INPUTVALUE;
		    datafmt.locale = NULL;
		    
		    if((retcode = ct_param(cmd, &datafmt, (CS_VOID *)&moneyvar, 
					   CS_SIZEOF(CS_MONEY), CS_UNUSED))  != CS_SUCCEED)
		    {
			ex_error("BuildRpcCommand: ct_param(money) failed");
			return retcode;
		    }
		    break;
	    }
	}
		    
	return retcode;
}

/*
** DoRpc()
**
** Type of function:
** 	rpc program internal api
**
** Purpose:
**	This routine passes the parameters and runs 
**	the sample RPC on the server.
**
** Parameters:
** 	connection	- Pointer to CS_CONNECTION structure.
**
** Return:
**	CS_SUCCEED if rpc was executed.
**	Otherwise a Client-Library failure code.
** 
*/

CS_STATIC CS_RETCODE 
DoRpc(arg, bufflen, d)
char *arg;
int bufflen;
example_data* d;
{
	CS_RETCODE	retcode;
	CS_SMALLINT	msg_id;
	CS_INT		res_type;	/* result type from ct_results */



  

        if ((retcode = BuildRpcCommand(arg, bufflen)) != CS_SUCCEED)
        {
                ex_error("DoRpc: BuildRpcCommand() failed");
                return retcode;
        }

	/*
	** Send the command to the server 
	*/
	if (ct_send(cmd) != CS_SUCCEED)
	{
                ex_error("DoCompute: ct_send() failed");
                return retcode;
	}
	
	/*
	** Process the results of the RPC.
	*/
	while ((retcode = ct_results(cmd, &res_type)) == CS_SUCCEED)
	{
		switch ((int)res_type)
		{
		case CS_ROW_RESULT:
		case CS_PARAM_RESULT:
		case CS_STATUS_RESULT:
			
			/*
			** All three of these result types are fetchable.
			** Since the result model for rpcs and rows have
			** been unified in the New Client-Library, we
			** will use the same routine to display them
			*/
			retcode = ex_fetch_data(cmd, d);
			if (retcode != CS_SUCCEED)
			{
				ex_error("DoRpc: ex_fetch_data() failed");
				return retcode;
			}
			break;

		case CS_MSG_RESULT:
			/*
			** 
			*/
			retcode = ct_res_info(cmd, CS_MSGTYPE,
					(CS_VOID *)&msg_id, CS_UNUSED, NULL);
			if (retcode != CS_SUCCEED)
			{
				ex_error("DoRpc: ct_res_info(msgtype) failed");
				return retcode;
			}
			fprintf(stdout, "ct_result returned CS_MSG_RESULT where msg id = %d.\n", msg_id);
			fflush(stdout);
			break;

		case CS_CMD_SUCCEED:
			/*
			** This means no rows were returned.
			*/
			break;

		case CS_CMD_DONE:
			/*
			** Done with result set.
			*/
			break;

		case CS_CMD_FAIL:
			/*
			** The server encountered an error while
			** processing our command.
			*/
			ex_error("DoRpc: ct_results returned CS_CMD_FAIL.");
			break;

		default:
			/*
			** We got something unexpected.
			*/
			ex_error("DoRpc: ct_results returned unexpected result type");
			return CS_FAIL;
		}
	}


	/*
	** We're done processing results. Let's check the
	** return value of ct_results() to see if everything
	** went ok.
	*/
	
	switch ((int)retcode)
	{
		case CS_END_RESULTS:
			/*
			** Everything went fine.
			*/
			retcode = CS_SUCCEED;
			break;

		case CS_FAIL:
			/*
			** Something failed happened.
			*/
			ex_error("DoRpc: ct_results() failed");
			break;

		default:
			/*
			** We got an unexpected return value.
			*/
			ex_error("DoRpc: ct_results returned unexpected result type");
			break;
	}
	
	return retcode;
}
